/*
   pravidla:
     1s=1000ms
     Uint32 SDL_GetTicks() vracia pocet ms od startu SDL,
     po zhruba 49 dnoch dojde k prekroceniu 32bit limitu
     a citac sa vynuluje
   uzitocne vzorce:
     rychlost  v=s/t [m/s]
               v=at [m/s]
     zrychlenie             a=s/t2 [m/s2]
     gravitacne zrychlenie  g=v/t [m/s2], g na Zemi=9.81 m/s2
                            v=gt
*/
// hlavickove subory *******************************************************
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mCamera.h"

// konstruktor *************************************************************
mCamera::mCamera()
{
 scr_w=0.0f; scr_h=0.0f; // pociatocne rozmery okna

 cam_reset(); // reset parametrov kamery

 collision_onoff=1; // detekcia kolizii je zapnuta
}

// destruktor **************************************************************
mCamera::~mCamera()
{

}

// reset pohladu kamery ****************************************************
void mCamera::cam_reset(void)
{
 // naplnenie matice kamery pociatocnymi hodnotami
 for (t_a=0;t_a<16;t_a++) matrix[t_a]=0.0f;
 matrix[0]=1.0f; matrix[5]=1.0f; matrix[10]=1.0f; matrix[15]=1.0f;
 // nastavenie parametrov kamery
 posx=-3.0f; posy=-1.8f; posz=2.0f;
 movx=0.0f; movy=0.0f; movz=0.0f;
 angx=0.0f; angy=0.0f; angz=0.0f;
 rotx=0.0f; roty=0.0f; rotz=0.0f;
 oldx=0.0f; oldy=0.0f; oldz=0.0f;
 targetx=0.0f; targety=0.0f; targetz=0.0f;
 speed=2.0f; slide=2.0f; alt=2.0f;
 acceleration=2.0f; gravity=9.81f;
 viewdist=5000.0f;
 FOV=60.0f;
 JumpPower=0.0f;

 // prepocet pre posun
 go_x=0.0f; go_y=0.0f; go_z=0.0f;

 // nastavenie faktorov pre prepocet posunu a rotacie
 rotx_factor=10.0f; roty_factor=10.0f; rotz_factor=32.0f;
 movx_factor=2.0f; movy_factor=2.0f; movz_factor=2.0f;

 // pociatocne zresetovanie posunu
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0; goFast=0;
 goJump=0;

 // strelba
 isFire1=0;
 isFire2=0;

 // sposob vypoctu pohybu
 movetype=0;

 // reset casu
 cam_resettime();
}

// reset casu **************************************************************
void mCamera::cam_resettime(void)
{
 // reset casu
 oldTime=SDL_GetTicks(); actTime=oldTime;
 difTime=0.0f;
 gravityTime=0.0f;

 ogl_fps=0.0f;
 ogl_frames=0.0f;
}

// reinicializacia kamery **************************************************
void mCamera::cam_reinit(GLfloat w,GLfloat h)
{
 glMatrixMode(GL_PROJECTION); // vyber projekcnej matice
 glLoadIdentity();            // zresetovanie projekcnej matice

 if (h==0.0f) h=1.0f;
 gluPerspective(FOV,w/h,0.1f,viewdist); // nastavenie perspektivneho premietania
 // FOV - zorny uhol kamery, w/h - aspect je pomer vysky a sirky zobrazovanej plochy
 // 1.0f - minimalna viditelna vzdialenost, viewdist - maximalna viditelna vzdialenost
 glViewport(0,0,(GLuint) w,(GLuint) h);  // nastavenie viditelnej plochy

 // TRANSFORMUJEME SCENU NIE KAMERU!
 glMatrixMode(GL_MODELVIEW); // vyber modelovej matice
 glLoadIdentity();           // zresetovanie modelovej matice

 cam_update(); // uprava parametrov kamery

 scr_w=w; scr_h=h; // ulozenie rozmerov okna
}

// aktualizacia nastaveni kamery *******************************************
void mCamera::cam_update(void)
{
 // ulozenie povodnej polohy
 oldx=posx; oldy=posy; oldz=posz;

 // SPECIALNY PRISTUP K ROTACII A NASLEDNEMU POSUNU KAMERY!
 glMatrixMode(GL_MODELVIEW);
 glLoadMatrixf(matrix);

 switch (movetype) {
  case 0:  // chodza
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,0.0f,1.0f,0.0f);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;

   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

   // nastavenie novej polohy
   posx+=matrix[0]*movx+0.0f*movy-matrix[2]*movz;
   posy+=matrix[4]*movx+1.0f*movy-matrix[6]*movz;
   posz+=matrix[8]*movx+0.0f*movy-matrix[10]*movz;
   break;
  case 1:  // let
   // ROTACIA
   glRotatef(rotx,matrix[0],matrix[4],matrix[8]);
   glRotatef(roty,matrix[1],matrix[5],matrix[9]);
   glRotatef(rotz,matrix[2],matrix[6],matrix[10]);
   // nastavenie noveho otocenia
   angx+=rotx; angy+=roty; angz+=rotz;

   glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

   // nastavenie novej polohy
   posx+=matrix[0]*movx+matrix[1]*movy-matrix[2]*movz;
   posy+=matrix[4]*movx+matrix[5]*movy-matrix[6]*movz;
   posz+=matrix[8]*movx+matrix[9]*movy-matrix[10]*movz;
   break;
  default: // dalsie typy pohybu...
   break;
 }

 // nastavenie smeroveho vektora kamery
 targetx=matrix[2];
 targety=matrix[6];
 targetz=matrix[10];
}

// posun kamery ************************************************************
void mCamera::cam_move(void)
{
 // cas...
 actTime=SDL_GetTicks();
 difTime=((GLfloat) (actTime-oldTime))/1000.0f;
 oldTime=actTime;

 // INFO: vypis casoveho okamihu uplynuteho medzi vykreslenim dvoch scen
 //fprintf(stderr,"%f\n",difTime);

 // pocitadlo FPS
 ogl_frames++;
 if (difTime>=0.001f) {
  ogl_fps=ogl_frames/difTime;
  ogl_frames=0.0f;
 }
/*
 if (goForward) go_z=speed*difTime;    // dopredu a dozadu
 else if (goBack) go_z=-speed*difTime;
      else go_z=0.0f;

 if (goRight) go_x=slide*difTime;      // vlavo a vpravo
 else if (goLeft) go_x=-slide*difTime;
      else go_x=0.0f;
*/
 // vylepseny sposob pohybu, ak su stlacene sucasne dve klavesy, avatar sa nehybe
 go_z=(((GLfloat) goForward)-((GLfloat) goBack))*speed*difTime; // dopredu a dozadu

 go_x=(((GLfloat) goRight)-((GLfloat) goLeft))*slide*difTime;   // vlavo a vpravo

 go_y=(((GLfloat) goUp)-((GLfloat) goDown))*alt*difTime;        // hore a dole

 // pri type pohybu 'chodza' posobi gravitacia
 if (movetype==0) {
  if (gravityTime==0.0f) { // ak je avatar na zemi
   if (goJump) {           // ak bola stlacena klavesa pre skok
    if (ogl_fps!=0.0f) JumpPower=sqrt(1.0f/ogl_fps); // vypocet sily skoku (zavisle od FPS!)
    else JumpPower=sqrt(1.0f);                       // kontrola na FPS=0!
   }
   else JumpPower=0.0f;    // inac nie je skok
  }

  // zvacsovanie sa casu trvania skoku/posobenia gravitacie
  gravityTime+=difTime;
  
  // vypocet klesania v zavislosti od skoku a gravitacie
  go_y=JumpPower-gravity*gravityTime/20.0f;

  // klesanie nemoze byt vacsie ako rozmer avatara, lebo nastanu chyby pri vypocte kolizie
  if (go_y<-1.0f) go_y=-1.0f;
 }

 if (goFast) { go_x*=4.0f; go_y*=1.0f; go_z*=4.0f; }

 movx=-go_x*movx_factor;
 movy=-go_y*movy_factor;
 movz=-go_z*movz_factor;

 // zrusenie dalsieho mozneho posunu, rotacie alebo skoku
 goForward=0; goBack=0; goUp=0; goDown=0; goRight=0; goLeft=0; goFast=0;
 goJump=0;
}

// rotacia kamery **********************************************************
void mCamera::cam_rotate(GLfloat x,GLfloat y,GLfloat z)
{
 rotx=x/rotx_factor; roty=y/roty_factor; rotz=z/rotz_factor;
}

// zap/vyp detekcie kolizii ************************************************
void mCamera::CollisionOnOff(void)
{
 if (collision_onoff==0) collision_onoff=1; else collision_onoff=0;

 SDL_Delay(200);
}

// matematicke operacie pre vypocet kolizii ********************************
#include "m3Dmath.h"

m3Dvector myTriangle[3];
m3Dvector myTriangleNormal;
m3Dvector myLine[2];
m3Dvector myTrianglePlanes[3];
float D1,D2,D3;

m3Dmath my_3Dmath;

// detekcia kolizie kamery s objektom sceny (trojuholnikom) ****************
int mCamera::cam_collision(GLfloat x1,GLfloat y1,GLfloat z1,
	                       GLfloat x2,GLfloat y2,GLfloat z2,
	                       GLfloat x3,GLfloat y3,GLfloat z3,
                           GLfloat tr_nx,GLfloat tr_ny,GLfloat tr_nz,
                           GLfloat tr_cn1,GLfloat tr_cn2, GLfloat tr_cn3, GLfloat tr_cn4,
                           GLfloat tr_cn5,GLfloat tr_cn6, GLfloat tr_cn7, GLfloat tr_cn8,
                           GLfloat tr_cn9,GLfloat tr_cn10,GLfloat tr_cn11,GLfloat tr_cn12)

{
 // vrcholy trojuholnika
 myTriangle[0].x=x1; myTriangle[0].y=y1; myTriangle[0].z=z1;
 myTriangle[1].x=x2; myTriangle[1].y=y2; myTriangle[1].z=z2;
 myTriangle[2].x=x3; myTriangle[2].y=y3; myTriangle[2].z=z3;

 // pozicia kamery
 myLine[1].x=-posx; myLine[1].y=-posy; myLine[1].z=-posz;

 // normala trojuholnika
 myTriangleNormal.x=tr_nx; myTriangleNormal.y=tr_ny; myTriangleNormal.z=tr_nz;

 // koeficienty rovnic koliznych rovin
 myTrianglePlanes[0].x=tr_cn1; myTrianglePlanes[0].y=tr_cn2;  myTrianglePlanes[0].z=tr_cn3;  D1=tr_cn4;
 myTrianglePlanes[1].x=tr_cn5; myTrianglePlanes[1].y=tr_cn6;  myTrianglePlanes[1].z=tr_cn7;  D2=tr_cn8;
 myTrianglePlanes[2].x=tr_cn9; myTrianglePlanes[2].y=tr_cn10; myTrianglePlanes[2].z=tr_cn11; D3=tr_cn12;
 
 if (my_3Dmath.CameraInTriangleS(myTriangle,myTriangleNormal,myLine[1],1.8f,
                                 myTrianglePlanes,D1,D2,D3)) {
  // pozicia kamery bude zmenena na zaklade kolizie s trojuholnikom, resp. jeho hranami
  posx=-my_3Dmath.ret_pos.x; posy=-my_3Dmath.ret_pos.y; posz=-my_3Dmath.ret_pos.z;

  // pad avatara bude zastaveny v pripade kolizie s "podlahou"
  if (myTriangleNormal.y>0.0f) gravityTime=0.0f;

  // nastala kolizia
  is_collision=1;
  
  return(1);
 }

 return(0);	
}
