// hlavickove subory *******************************************************
// systemove funkcie
#include <math.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// GLEXT kniznica
#include <GL/glext.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// .h subor
#include "mExt.h"

#define REFLECT_SIZE 256
#define REFRACT_SIZE 256
#define COMBINE_SIZE 256

// konstruktor *************************************************************
mExt::mExt()
{
 glActiveTextureARB=NULL;
 glMultiTexCoord2fARB=NULL;

 maxTexelUnits=1; // kazda karta ma aspon 1 texturovaciu jednotku

 glCombinerParameterfvNV=NULL;
 glCombinerParameteriNV=NULL;
 glCombinerInputNV=NULL;
 glCombinerOutputNV=NULL;
 glFinalCombinerInputNV=NULL;
}

// destruktor **************************************************************
mExt::~mExt()
{

}

// inicializacia rozsireni *************************************************
int mExt::Ext_init()
{
 glGetString(GL_EXTENSIONS); // DOROBIT! testovanie rozsireni

 glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB,&maxTexelUnits);

 glActiveTextureARB=(PFNGLACTIVETEXTUREARBPROC) SDL_GL_GetProcAddress("glActiveTextureARB");
 glMultiTexCoord2fARB=(PFNGLMULTITEXCOORD2FARBPROC) SDL_GL_GetProcAddress("glMultiTexCoord2fARB");

 fprintf(stderr,"\nextension used:\n");
 if (glActiveTextureARB==NULL) fprintf(stderr,"glActiveTextureARB=NULL\n");
 else fprintf(stderr,"glActiveTextureARB=1\n");
 if (glMultiTexCoord2fARB==NULL) fprintf(stderr,"glMultiTexCoord2fARB=NULL\n");
 else fprintf(stderr,"glMultiTexCoord2fARB=1\n");

 glCombinerParameterfvNV=(PFNGLCOMBINERPARAMETERFVNVPROC) SDL_GL_GetProcAddress("glCombinerParameterfvNV");
 glCombinerParameteriNV=(PFNGLCOMBINERPARAMETERINVPROC) SDL_GL_GetProcAddress("glCombinerParameteriNV");
 glCombinerInputNV=(PFNGLCOMBINERINPUTNVPROC) SDL_GL_GetProcAddress("glCombinerInputNV");
 glCombinerOutputNV=(PFNGLCOMBINEROUTPUTNVPROC) SDL_GL_GetProcAddress("glCombinerOutputNV");
 glFinalCombinerInputNV=(PFNGLFINALCOMBINERINPUTNVPROC) SDL_GL_GetProcAddress("glFinalCombinerInputNV");

 if (glCombinerParameterfvNV==NULL) fprintf(stderr,"glCombinerParameterfvNV=NULL\n");
 else fprintf(stderr,"glCombinerParameterfvNV=1\n");
 if (glCombinerParameteriNV==NULL) fprintf(stderr,"glCombinerParameteriNV=NULL\n");
 else fprintf(stderr,"glCombinerParameteriNV=1\n");
 if (glCombinerInputNV==NULL) fprintf(stderr,"glCombinerInputNV=NULL\n");
 else fprintf(stderr,"glCombinerInputNV=1\n");
 if (glCombinerOutputNV==NULL) fprintf(stderr,"glCombinerOutputNV=NULL\n");
 else fprintf(stderr,"glCombinerOutputNV=1\n");
 if (glFinalCombinerInputNV==NULL) fprintf(stderr,"glFinalCombinerInputNV=NULL\n");
 else fprintf(stderr,"glFinalCombinerInputNV=1\n");

 return(0);
}

// multitexturovana kocka **************************************************
void mExt::MultiTCube(GLuint T1,GLuint T2,GLfloat size)
{
 // ulozenie potrebnych atributov
 glPushAttrib(GL_LIGHTING_BIT|GL_TEXTURE_BIT|GL_FOG_BIT);
 glDisable(GL_LIGHTING);   // vypnutie osvetlenia
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania
 glDisable(GL_FOG);        // vypnutie hmly
 glDisable(GL_BLEND);      // vypnutie priesvitnosti

 // texturovacia jednotka0
 // texture-combiner0=GL_REPLACE
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glEnable(GL_TEXTURE_2D); // zapnutie texturovania pre 1. jednotku
 glBindTexture(GL_TEXTURE_2D,T1);
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
 glTexEnvf(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,GL_REPLACE);
 // texturovacia jednotka1
 // texture-combiner1=GL_ADD (multitexture-equivalent to ONE blending)
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glEnable(GL_TEXTURE_2D); // zapnutie texturovania pre 2. jednotku
 glBindTexture(GL_TEXTURE_2D,T2);
 glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_COMBINE_EXT);
 glTexEnvf(GL_TEXTURE_ENV,GL_COMBINE_RGB_EXT,GL_ADD);

 glColor4f(1.0f,1.0f,1.0f,1.0f); // farba
 glBegin(GL_QUADS);              // kocka
  glNormal3f(0.0f,0.0f,1.0f);    // predna stena
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,0.0f);
  glVertex3f(-size,-size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,0.0f);
  glVertex3f(size,-size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,1.0f);
  glVertex3f(size,size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,1.0f);
  glVertex3f(-size,size,size);

  glNormal3f(1.0f,0.0f,0.0f);    // prava stena
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,0.0f);
  glVertex3f(size,-size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,0.0f);
  glVertex3f(size,-size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,1.0f);
  glVertex3f(size,size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,1.0f);
  glVertex3f(size,size,size);
 glEnd();

 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_2D); // vypnutie texturovania pre 2. jednotku
 glActiveTextureARB(GL_TEXTURE0_ARB);

 glBegin(GL_QUADS);              // kocka
  glNormal3f(0.0f,0.0f,-1.0f);   // zadna stena
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,0.0f);
  glVertex3f(size,-size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,0.0f);
  glVertex3f(-size,-size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,1.0f);
  glVertex3f(-size,size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,1.0f);
  glVertex3f(size,size,-size);

  glNormal3f(-1.0f,0.0f,0.0f);   // lava stena
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,0.0f);
  glVertex3f(-size,-size,-size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,0.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,0.0f);
  glVertex3f(-size,-size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,1.0f,1.0f);
  glVertex3f(-size,size,size);
  glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f,1.0f);
  glMultiTexCoord2fARB(GL_TEXTURE1_ARB,0.0f,1.0f);
  glVertex3f(-size,size,-size);
 glEnd();

 glBindTexture(GL_TEXTURE_2D,T2);

 glBegin(GL_QUADS);              // kocka
  glNormal3f(0.0f,-1.0f,0.0f);   // dolna stena
  glTexCoord2f(0.0f,0.0f);
  glVertex3f(-size,-size,-size);
  glTexCoord2f(1.0f,0.0f);
  glVertex3f(size,-size,-size);
  glTexCoord2f(1.0f,1.0f);
  glVertex3f(size,-size,size);
  glTexCoord2f(0.0f,1.0f);
  glVertex3f(-size,-size,size);

  glNormal3f(0.0f,1.0f,0.0f);    // horna stena
  glTexCoord2f(0.0f,0.0f);
  glVertex3f(-size,size,size);
  glTexCoord2f(1.0f,0.0f);
  glVertex3f(size,size,size);
  glTexCoord2f(1.0f,1.0f);
  glVertex3f(size,size,-size);
  glTexCoord2f(0.0f,1.0f);
  glVertex3f(-size,size,-size);
 glEnd();

 glPopAttrib(); // vyber ulozenych atributov
}

void create_combine_texture(int width,int height,unsigned char *data)
{
 for (int y=0;y<height;y++) {
  for (int x=0;x<width;x++) {
   float fx=x/(float) width-0.5f;
   float fy=y/(float) height-0.5f;
   float r=sqrt(fx*fx+fy*fy);

   unsigned char ds=(unsigned char) (64*cos(300.0f*r)*exp(-r*5.0f));
   ds+=(unsigned char) (32*cos(150.0f*(fx+fy)));
   ds+=(unsigned char) (16*cos(140.0f*(fx*0.85f-fy)));

   unsigned char dt=(unsigned char) (64*sin(300.0f*r)*exp(-r*5.0f));
   dt+=(unsigned char) (32*sin(150.0f*(fx+fy)));
   dt+=(unsigned char) (16*sin(140.0f*(fx*0.85f-fy)));

   *data++=ds;
   *data++=dt;
  }
 }
}

int mExt::water_init(void) {
 glGenTextures(1,&reflect_tex);
 glBindTexture(GL_TEXTURE_2D,reflect_tex);
 glTexParameteri(GL_TEXTURE_2D,GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,REFLECT_SIZE,REFLECT_SIZE,0,GL_RGB,GL_UNSIGNED_BYTE,NULL);

 glGenTextures(1,&refract_tex);
 glBindTexture(GL_TEXTURE_2D,refract_tex);
 glTexParameteri(GL_TEXTURE_2D,GL_GENERATE_MIPMAP_SGIS,GL_TRUE);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);
 glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,REFRACT_SIZE,REFRACT_SIZE,0,GL_RGB,GL_UNSIGNED_BYTE,NULL);

 glGenTextures(1,&combine_tex);
 glBindTexture(GL_TEXTURE_2D,combine_tex);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
 unsigned char *combine_data=new unsigned char[COMBINE_SIZE*COMBINE_SIZE*2];
 create_combine_texture(COMBINE_SIZE,COMBINE_SIZE,combine_data);
 glTexImage2D(GL_TEXTURE_2D,0,GL_DSDT_NV,COMBINE_SIZE,COMBINE_SIZE,0,GL_DSDT_NV,GL_UNSIGNED_BYTE,combine_data);

 reflect_clip_equation[0]=0.0f;
 reflect_clip_equation[1]=1.0f;
 reflect_clip_equation[2]=0.0f;
 reflect_clip_equation[3]=0.0f;

 refract_clip_equation[0]=0.0f;
 refract_clip_equation[1]=-1.0f;
 refract_clip_equation[2]=0.0f;
 refract_clip_equation[3]=0.0f;

 k=0.0f;

 return(0);
}

void mExt::start_prepare_water_texture(GLuint w,GLuint h)
{
 // ulozenie aktualnej velkosti okna
 scr_w=w; scr_h=h;

 // zapnutie orezavacieho testu
 glEnable(GL_SCISSOR_TEST);

 // nastavenie velkosti okna pre pripravu textury
 glViewport(0,0,REFLECT_SIZE,REFLECT_SIZE);
 glScissor(0,0,REFLECT_SIZE,REFLECT_SIZE);

 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
}

void mExt::continue_prepare_water_texture(void)
{
 // bude upravena textura odrazu
 glBindTexture(GL_TEXTURE_2D,reflect_tex);
 // udajova cast textury sa naplni farbami pixelov
 glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,REFLECT_SIZE,REFLECT_SIZE);

 // nastavenie velkosti okna pre pripravu textury
 // nemusi sa menit pokial REFRACT_SIZE=REFLECT_SIZE
 //glViewport(0,0,REFRACT_SIZE,REFRACT_SIZE);
 //glScissor(0,0,REFRACT_SIZE,REFRACT_SIZE);

 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
}

void mExt::end_prepare_water_texture(void)
{
 // bude upravena textura prechodu
 glBindTexture(GL_TEXTURE_2D,refract_tex);
 // udajova cast textury sa naplni farbami pixelov
 glCopyTexSubImage2D(GL_TEXTURE_2D,0,0,0,0,0,REFRACT_SIZE,REFRACT_SIZE);

 // nastavenie povodnej velkosti okna pre rendering sceny
 glViewport(0,0,scr_w,scr_h);

 // vypnutie orezavacieho testu
 glDisable(GL_SCISSOR_TEST);
}

void mExt::render_water(GLfloat ctime)
{
 glGetFloatv(GL_MODELVIEW_MATRIX,modelview_mat);

 k+=ctime;
 // INFO: casovy posun
 //fprintf(stderr,"%f\n",k);

 plane_s[0]=1.0f; plane_s[1]=0.0f; plane_s[2]=0.0f; plane_s[3]=0.0f;
 plane_t[0]=0.0f; plane_t[1]=1.0f; plane_t[2]=0.0f; plane_t[3]=0.0f;
 plane_r[0]=0.0f; plane_r[1]=0.0f; plane_r[2]=1.0f; plane_r[3]=0.0f;
 plane_q[0]=0.0f; plane_q[1]=0.0f; plane_q[2]=0.0f; plane_q[3]=1.0f;

 // vyber 1. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE0_ARB);
 glTexEnvi(GL_TEXTURE_SHADER_NV,GL_SHADER_OPERATION_NV,GL_TEXTURE_2D);

 // vyber 2. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glTexEnvi(GL_TEXTURE_SHADER_NV,GL_SHADER_OPERATION_NV,GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV);
 glTexEnvi(GL_TEXTURE_SHADER_NV,GL_PREVIOUS_TEXTURE_INPUT_NV,GL_TEXTURE0_ARB);
 v[0]=cos(k)*0.02; v[1]=-sin(k)*0.02; v[2]=sin(k)*0.02; v[3]=cos(k)*0.02;
 glTexEnvfv(GL_TEXTURE_SHADER_NV,GL_OFFSET_TEXTURE_2D_MATRIX_NV,v);

 // vyber 3. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE2_ARB);
 glTexEnvi(GL_TEXTURE_SHADER_NV,GL_SHADER_OPERATION_NV,GL_OFFSET_PROJECTIVE_TEXTURE_2D_NV);
 glTexEnvi(GL_TEXTURE_SHADER_NV,GL_PREVIOUS_TEXTURE_INPUT_NV,GL_TEXTURE0_ARB);
 v[0]=cos(k)*0.01; v[1]=-sin(k)*0.01; v[2]=sin(k)*0.01; v[3]=cos(k)*0.01;
 glTexEnvfv(GL_TEXTURE_SHADER_NV,GL_OFFSET_TEXTURE_2D_MATRIX_NV,v);

 // vyber 1. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE0_ARB);

 v[0]=0.6f; v[1]=0.6f; v[2]=0.6f; v[3]=1.0f;
 glCombinerParameterfvNV(GL_CONSTANT_COLOR0_NV,v);
 v[0]=0.3f; v[1]=0.3f; v[2]=0.3f; v[3]=1.0f;
 glCombinerParameterfvNV(GL_CONSTANT_COLOR1_NV,v);

 glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV,1);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_A_NV,GL_TEXTURE1_ARB,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_B_NV,GL_CONSTANT_COLOR0_NV,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_C_NV,GL_TEXTURE2_ARB,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerInputNV(GL_COMBINER0_NV,GL_RGB,GL_VARIABLE_D_NV,GL_CONSTANT_COLOR1_NV,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glCombinerOutputNV(GL_COMBINER0_NV,GL_RGB,GL_DISCARD_NV,GL_DISCARD_NV,GL_SPARE0_NV,GL_NONE,GL_NONE,GL_FALSE,GL_FALSE,GL_FALSE);

 glFinalCombinerInputNV(GL_VARIABLE_A_NV,GL_SPARE0_NV,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_B_NV,GL_ZERO,GL_UNSIGNED_INVERT_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_C_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_D_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_E_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);
 glFinalCombinerInputNV(GL_VARIABLE_F_NV,GL_ZERO,GL_UNSIGNED_IDENTITY_NV,GL_RGB);

 // zapnutie texturovacieho shadera
 glEnable(GL_TEXTURE_SHADER_NV);
 // zapnutie
 glEnable(GL_REGISTER_COMBINERS_NV);

 // reflexna textura sa spracuje v 2. texturovacej jednotke
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glBindTexture(GL_TEXTURE_2D,reflect_tex);
 glTexGenfv(GL_S,GL_EYE_PLANE,plane_s);
 glTexGenfv(GL_T,GL_EYE_PLANE,plane_t);
 glTexGenfv(GL_R,GL_EYE_PLANE,plane_r);
 glTexGenfv(GL_Q,GL_EYE_PLANE,plane_q);
 glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_R,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_Q,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glEnable(GL_TEXTURE_GEN_S);
 glEnable(GL_TEXTURE_GEN_T);
 glEnable(GL_TEXTURE_GEN_R);
 glEnable(GL_TEXTURE_GEN_Q);
 glMatrixMode(GL_TEXTURE);
 glLoadIdentity();
 glTranslatef(0.5f,0.5f,0.0f);
 glScalef(0.5f,0.5f,0.0f);
 gluPerspective(60.0f,720.0f/486.0f,1.0f,5000.0f);
 glMultMatrixf(modelview_mat);
 glScalef(1.0f,-1.0f,1.0f);
 glMatrixMode(GL_MODELVIEW);

 // refrakcna textura sa spracuje v 3. texturovacej jednotke
 glActiveTextureARB(GL_TEXTURE2_ARB);
 glBindTexture(GL_TEXTURE_2D,refract_tex);
 glTexGenfv(GL_S,GL_EYE_PLANE,plane_s);
 glTexGenfv(GL_T,GL_EYE_PLANE,plane_t);
 glTexGenfv(GL_R,GL_EYE_PLANE,plane_r);
 glTexGenfv(GL_Q,GL_EYE_PLANE,plane_q);
 glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_R,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glTexGeni(GL_Q,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
 glEnable(GL_TEXTURE_GEN_S);
 glEnable(GL_TEXTURE_GEN_T);
 glEnable(GL_TEXTURE_GEN_R);
 glEnable(GL_TEXTURE_GEN_Q);
 glMatrixMode(GL_TEXTURE);
 glLoadIdentity();
 glTranslatef(0.5f,0.5f,0.0f);
 glScalef(0.5f,0.5f,0.0f);
 gluPerspective(60.0f,720.0f/486.0f,1.0f,5000.0f);
 glMultMatrixf(modelview_mat);
 glMatrixMode(GL_MODELVIEW);

 // vyber 1. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE0_ARB);
 // zvolenie patricnej textury
 glBindTexture(GL_TEXTURE_2D,combine_tex);

 // vykreslenie plochy vody
 glBegin(GL_QUADS);
  glTexCoord2f(0,0); glVertex3f(-200.0f,0.0f,200.0f);
  glTexCoord2f(1,0); glVertex3f( 200.0f,0.0f,200.0f);
  glTexCoord2f(1,1); glVertex3f( 200.0f,0.0f,-200.0f);
  glTexCoord2f(0,1); glVertex3f(-200.0f,0.0f,-200.0f);
 glEnd();

 // vypnutie generacie koordinatov textur pre 2. jednotku
 glActiveTextureARB(GL_TEXTURE1_ARB);
 glDisable(GL_TEXTURE_GEN_S);
 glDisable(GL_TEXTURE_GEN_T);
 glDisable(GL_TEXTURE_GEN_R);
 glDisable(GL_TEXTURE_GEN_Q);

 // vypnutie generacie koordinatov textur pre 3. jednotku
 glActiveTextureARB(GL_TEXTURE2_ARB);
 glDisable(GL_TEXTURE_GEN_S);
 glDisable(GL_TEXTURE_GEN_T);
 glDisable(GL_TEXTURE_GEN_R);
 glDisable(GL_TEXTURE_GEN_Q);

 // vyber 1. texturovacej jednotky
 glActiveTextureARB(GL_TEXTURE0_ARB);

 // vypnutie
 glDisable(GL_REGISTER_COMBINERS_NV);
 // vypnutie texturovacieho shadera
 glDisable(GL_TEXTURE_SHADER_NV);
}

/*
  tento kod vznikol po uprave zverejneneho kodu autora:

  Alexander Zaprjagaev
  frustum@frustum.tomsk.ru
  http://frustum.tomsk.ru
*/
